@@ -1,8 +1,6 @@
 """Virtualreality Utility Module"""
 from ..logging import log
 
-logger = log.setup_custom_logger(name=__name__, 
-    level="INFO",
-    file="../logs/util.log")
+logger = log.setup_custom_logger(name=__name__, level="INFO", file="../logs/util.log")
 
-logger.debug("created util.log log file")
\ No newline at end of file
+logger.debug("created util.log log file")
