@@ -5,19 +5,21 @@ import threading
 import time
 from asyncio.streams import StreamReader
 from typing import Sequence, Dict
+import struct
 
-import cv2
 import numpy as np
 import serial.threaded
 from pykalman import KalmanFilter
 
 try:
+    import cv2
     from displayarray import read_updates
+    from displayarray import display
 
     HAVE_VOD = True
 
 except Exception as e:
-    print(f"failed to import displayarray, reason: {e}")
+    print(f"failed to import displayarray and/or opencv, reason: {e}")
     print("camera based tracking methods will not be available")
     HAVE_VOD = False
 
@@ -290,6 +292,8 @@ class SerialReaderFactory(serial.threaded.LineReader):
             protocol.lastRead # contains the last incoming message from serial
             protocol.write_line(single_line_text) # used to write a single line to serial
     """
+    TERMINATOR = b'\n'
+
 
     def __init__(self):
         """Create the SerialReaderFactory."""
@@ -312,6 +316,52 @@ class SerialReaderFactory(serial.threaded.LineReader):
             f"SerialReaderFactory: port {repr(self.transport.serial.port)} closed {repr(exc)}"
         )
 
+class SerialReaderBinary(serial.threaded.Packetizer):
+    """
+    Read binary packets from serial port. Packets are expected to be terminated
+    with a TERMINATOR byte (null byte by default).
+
+    The class also keeps track of the transport.
+    """
+
+    TERMINATOR = b'\t\r\n'
+    ENCODING = 'utf-8'
+    UNICODE_HANDLING = 'replace'
+
+    def __init__(self, struct_type='f', struct_len=13, type_len=4):
+        super().__init__()
+        self._last_packet = None
+        self._struct_form = struct_type*struct_len
+        self._struct_len = struct_len*type_len
+
+    def __call__(self):
+        return self
+
+    @property
+    def last_read(self):
+        """Get the readonly last read."""
+        return self._last_packet
+
+    def connection_lost(self, exc):
+        """Notify the user that the connection was lost."""
+        print(
+            f"SerialReaderBinary: port {repr(self.transport.serial.port)} closed {repr(exc)}"
+        )
+
+    def handle_packet(self, packet):
+        """Process packets - to be overridden by subclassing"""
+        # print (repr(packet))
+        if len(packet) == self._struct_len:
+            self._last_packet = struct.unpack_from(self._struct_form, packet)
+
+    def write_line(self, text):
+        """
+        Write text to the transport. ``text`` is a Unicode string and the encoding
+        is applied before sending ans also the newline is append.
+        """
+        # + is not the best choice but bytes does not support % or .format in py3 and we want a single write call
+        self.transport.write(text.encode(self.ENCODING, self.UNICODE_HANDLING) + self.TERMINATOR)
+
 
 def cnt_2_x_y_w(cnt):
     nc = cnt.reshape((cnt.shape[0], 2))
@@ -340,18 +390,17 @@ class BlobTracker(threading.Thread):
         *,
         focal_length_px=490,
         ball_radius_cm=2,
-        color_masks={},
+        color_masks=[],
     ):
         """
         Create a blob tracker.
 
         example usage:
-        >>> tracker = BlobTracker(color_masks={
-        ...         'tracker1':{
-        ...             'h':(98, 10),
-        ...             's':(200, 55),
-        ...             'v':(250, 32)
-        ...         },
+        >>> tracker = BlobTracker(color_masks=[
+                ColorRange(98, 10, 200, 55, 250, 32),
+                ColorRange(68, 15, 135, 53, 255, 50),
+                ColorRange(68, 15, 135, 53, 255, 50),
+                                ],
         ... })
 
         >>> with tracker:
@@ -374,7 +423,7 @@ class BlobTracker(threading.Thread):
             )
 
         super().__init__()
-        self._vs = read_updates(cam_index)
+        self._vs = cv2.VideoCapture(cam_index)
 
         time.sleep(2)
 
@@ -388,20 +437,20 @@ class BlobTracker(threading.Thread):
         frame, self.can_track = self._try_get_frame()
 
         if not self.can_track:
-            self._vs.end()
+            self._vs.release()
             self._vs = None
             raise RuntimeError("invalid video source")
 
         self.frame_height, self.frame_width, _ = frame.shape
-        self.markerMasks = color_masks
+        self.markerMasks = np.array([[i.hue_center, i.hue_range, i.sat_center, i.sat_range, i.val_center, i.val_range] for i in color_masks], dtype=np.float32)
 
-        self.poses = np.zeros((len(self.markerMasks), 3))
+        self.poses = np.zeros((self.markerMasks.shape[0], 3))
         self.blobs = []
 
         self.kalmanFilterz = []
         self.kalmanFilterz2 = []
 
-        for i in range(len(self.markerMasks)):
+        for i in range(self.markerMasks.shape[0]):
             self.blobs.append(None)
             self.kalmanFilterz.append([False, None])
             self.kalmanFilterz2.append([False, None])
@@ -414,16 +463,18 @@ class BlobTracker(threading.Thread):
         self.pose_que = queue.Queue()
 
     def _try_get_frame(self):
-        self._vs.update()
-        try:
-            frame = self._vs.frames[str(self.cam_index)][0]
-            can_track = True
-            if frame is not self.last_frame:
-                self.time_of_last_frame = time.time()
-        except Exception as e:
-            print("_try_get_frame() failed with:", repr(e), self._vs.frames)
-            frame = None
-            can_track = False
+        # self._vs.update()
+        # try:
+        #     frame = self._vs.frames[str(self.cam_index)][0]
+        #     can_track = True
+        #     if frame is not self.last_frame:
+        #         self.time_of_last_frame = time.time()
+        # except Exception as e:
+        #     print("_try_get_frame() failed with:", repr(e), self._vs.frames)
+        #     frame = None
+        #     can_track = False
+        can_track, frame = self._vs.read()
+
         return frame, can_track
 
     def run(self):
@@ -464,21 +515,13 @@ class BlobTracker(threading.Thread):
         if self.alive:
             frame, self.can_track = self._try_get_frame()
 
-            for key, mask_range in enumerate(self.markerMasks.items()):
+            for key in range(self.markerMasks.shape[0]):
                 if self.can_track:
-                    hc, hr = mask_range[1]["h"]
-
-                    sc, sr = mask_range[1]["s"]
-
-                    vc, vr = mask_range[1]["v"]
-                    color_high = [hc + hr, sc + sr, vc + vr]
-                    color_low = [hc - hr, sc - sr, vc - vr]
-
-                    blurred = cv2.GaussianBlur(frame, (3, 3), 0)
+                    blurred = cv2.GaussianBlur(frame, (9, 9), 0)
 
                     hsv = cv2.cvtColor(blurred, cv2.COLOR_BGR2HSV)
 
-                    mask = cv2.inRange(hsv, tuple(color_low), tuple(color_high))
+                    mask = cv2.inRange(hsv, self.markerMasks[key][::2] - self.markerMasks[key][1::2], self.markerMasks[key][::2] + self.markerMasks[key][1::2])
 
                     temp = cv2.findContours(
                         mask.copy(), cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE
@@ -569,7 +612,7 @@ class BlobTracker(threading.Thread):
             self.stop()
 
             if self._vs is not None:
-                self._vs.end()
+                self._vs.release()
                 self._vs = None
 
     def __enter__(self):
